"""Module to store Ansi configurations.
In certain functions of the snakypy package, functions with named
parameters "foreground", "background" and "sgr" are found, in which you can receive a
certain foreground or background color, and font style settings in Ansi.
"""
import sys


def return_ansi_or_not(value):
    """This function checks if the operating system is Windows, and
    if an empty value is returned for each variable that represents a
    color in Ansi.

    Arguments:
        **value {str}** -- Receives a string in the form of Ansi color.

    Returns:
        **[str]** -- It will return an Ansi scape code, or an empty string.
    """
    if sys.platform.startswith('win'):
        return ''
    return value


NONE = return_ansi_or_not('\x1b[0m')
"""Variable to reset Ansi color settings."""


class FG:
    """
    Class FG (foreground) that receives colors and ANSI settings in each of
    the global variables below to be applied in texts.
    """

    BLACK = return_ansi_or_not('\x1b[30m')
    MAGENTA = return_ansi_or_not('\x1b[95m')
    BLUE = return_ansi_or_not('\x1b[94m')
    GREEN = return_ansi_or_not('\x1b[92m')
    RED = return_ansi_or_not('\x1b[91m')
    YELLOW = return_ansi_or_not('\x1b[93m')
    CYAN = return_ansi_or_not('\x1b[96m')
    WHITE = return_ansi_or_not('\x1b[97m')

    WARNING = return_ansi_or_not(f'{YELLOW}⚠ ')
    ERROR = return_ansi_or_not(f'{RED}✖ ')
    FINISH = return_ansi_or_not(f'{GREEN}✔ ')
    QUESTION = return_ansi_or_not(f'{CYAN}➜ ')


class BG:
    """
    Class BG (background) that receives colors and ANSI settings in each
    of the global variables below to be applied in texts.
    """

    BLACK = return_ansi_or_not('\x1b[40m')
    MAGENTA = return_ansi_or_not('\x1b[105m')
    BLUE = return_ansi_or_not('\x1b[104m')
    GREEN = return_ansi_or_not('\x1b[102m')
    RED = return_ansi_or_not('\x1b[101m')
    YELLOW = return_ansi_or_not('\x1b[103m')
    CYAN = return_ansi_or_not('\x1b[106m')
    WHITE = return_ansi_or_not('\x1b[107m')

    WARNING = return_ansi_or_not(f'{YELLOW}⚠ ')
    ERROR = return_ansi_or_not(f'{RED}✖ ')
    FINISH = return_ansi_or_not(f'{GREEN}✔ ')
    QUESTION = return_ansi_or_not(f'{CYAN}➜ ')


class SGR:
    """
    SGR class that receives effects for text such as underline, blink, etc.
    """

    BOLD = return_ansi_or_not('\x1b[1m')
    ITALIC = return_ansi_or_not('\x1b[3m')
    UNDERLINE = return_ansi_or_not('\x1b[4m')
    SLOW_BLINK = return_ansi_or_not('\x1b[5m')
    RAPID_BLINK = return_ansi_or_not('\x1b[6m')
    REVERSE_COLOR = return_ansi_or_not('\x1b[7m')


__all__ = ['NONE', 'FG', 'BG', 'SGR']



from snakypy.utils.calc import BMI


def bmi(sex, weight, height):
    """
    This function is responsible for calling the class of body mass index (BMI).

    >>> from snakypy.calc import bmi
    >>> bmi('m', 60, 1.73)

    **output:**

    .. code-block:: shell

        'Under weight.'

    Arguments:
        **sex {str}** -- You must receive a string with a single character. \
                     The string must be either "**m**" for "male" or "**f**" for "female"

        **weight {float}** -- You should receive a float, this is where you put the \
                          weight. **E.g:** 60.4

        **height {float}** -- You should receive a float, this is where you put the \
                          height. **E.g:** 1.73

    Returns:
        **[object]** -- The return will be a string informing the result or a false Boolean, \
                    if the arguments are wrong.
    """

    return BMI(sex, weight, height).main()


def percentage(per, whole, *, operation=None, log=False):
    """
    This function will calculate a whole with a certain percentage value.

    >>> from snakypy.calc import percentage
    >>> percentage(25, 500)
    >>> percentage(25, 500, log=True)
    >>> percentage(25, 500, operation='+')
    >>> percentage(25, 500, operation='-')
    >>> percentage(25, 500, operation='+', log=True)
    >>> percentage(25, 500, operation='-', log=True)

    **output:**

    .. code-block:: shell

        125
        '>> 25% of 500 = 125.00'
        625.00
        375.00
        '>> 500 + 25% = 625.00'
        '>> 500 - 25% = 375.00'

    Arguments:

        **per {float}** -- Input argument that should receive a float representing \
                       a certain percentage value.

        **whole {float}** -- Input argument that should receive a float representing \
                         total value.

    Keyword Arguments:

        **operation {str}** -- This named parameter must receive two types of string \
                           values. One is the plus sign ('+') that will add the total \
                           to the imposed percentage. The other is the minus sign ('-'), \
                           which subtracts the total value from the imposed percentage \
                           value. (default: {None})

        **log {bool}** -- If this parameter is of the value True, then the return will be \
                     a custom string with the percent signs and the operation. \
                     (default: {False})
    """

    option = {
        '+': lambda: whole + (whole * (per / 100)),
        '-': lambda: whole - (whole * (per / 100))
    }.get(operation, lambda: whole * (per / 100))()
    if log:
        if operation == '+':
            return f'>> {whole} + {per}% = {option:.2f}'
        elif operation == '-':
            return f'>> {whole} - {per}% = {option:.2f}'
        return f'>> {per}% of {whole} = {option:.2f}'
    return option


__all__ = ['bmi', 'percentage']



class BMI:
    """Class that will calculate the body mass index."""

    def __init__(self, sex, weight, height):
        self.sex = sex
        self.weight = weight
        self.height = height
        self.reg_male = {'V1': 20.7, 'V2': 26.4, 'V3': 27.8, 'V4': 31.1}
        self.reg_female = {'V1': 19.1, 'V2': 25.8, 'V3': 27.3, 'V4': 32.3}

    def calc_bmi(self):
        """
        This method will perform the body mass index calculation.
        Applied within the "calc_bmi" method.
        """
        try:
            w = float(self.validate_hw(self.weight, 0, 350))
            h = float(self.validate_hw(self.height, 0, 3))
            return w / (h * h)
        except Exception:
            return False

    def validate_sex(self):
        """Method to validate the person's gender.
        """
        if self.sex == 'm' or self.sex == 'f':
            return self.sex
        return False

    @staticmethod
    def validate_hw(hw, value1, value2):
        """
        This is a static method for validating weight and height.
        The height cannot be greater than 3 meters and neither the weight
        above 350, nor can both be zeroed or negative
        """
        if float(hw) <= value1 or float(hw) > value2:
            return False
        return hw

    @staticmethod
    def reply(result_bmi, reg):
        """
        The purpose of this statistical method is to receive the calculation
        of the body mass index already done, and to compare it with the
        statistical data informed in the initializer method of the class.
        """
        if result_bmi < reg['V1']:
            return 'Under weight.'
        elif reg['V1'] <= result_bmi < reg['V2']:
            return 'Normal weight.'
        elif reg['V2'] <= result_bmi < reg['V3']:
            return 'Marginally overweight.'
        elif reg['V3'] <= result_bmi < reg['V4']:
            return 'Overweight.'
        elif result_bmi > reg['V4']:
            return 'Obesity.'

    def main(self):
        if self.calc_bmi():
            if self.validate_sex() == 'm':
                return self.reply(self.calc_bmi(), self.reg_male)
            elif self.validate_sex() == 'f':
                return self.reply(self.calc_bmi(), self.reg_female)
        return False



from functools import wraps
from snakypy.utils.exceptions import NotSupportWindows


def use_unix_system(func):
    """
    A decorator to force a function or method to run on Unix systems only.
    """
    from sys import platform

    @wraps(func)
    def wrapper(*args, **kwargs):
        # Linux: startswith('linux')
        # OS X: startswith('darwin')
        # Windows: startswith('win')
        if platform.startswith('win'):
            msg = 'Invalid operating system "Windows". ' \
                  f'This function "{func.__name__}" is only compatible with ' \
                  'unix-based systems.'
            raise NotSupportWindows(msg)
        return func(*args, **kwargs)
    return wrapper


# DEPRECATED!
# def denying_win(*args, os='win'):
#     from sys import platform
#
#     # Linux: 'linux'
#     # OS X: 'darwin'
#     # Windows: 'win'
#
#     if args:
#         if (args != '') and platform.startswith(os):
#             raise Exception('>>> You cannot activate the color using Windows OS.')
#     else:
#         if platform.startswith(os):
#             msg = 'Invalid operating system (Windows). ' \
#                   f'This function "{__name__}" is compatible with ' \
#                   '"Linux" and "Mac OS X" systems only.'
#             raise Exception(msg)


__all__ = ['use_unix_system']



import snakypy.file
import snakypy.path
import snakypy.json
import snakypy.utils
import snakypy.console
import snakypy.ansi
import snakypy.calc
from snakypy.utils import catch
from snakypy.utils import stiff
from snakypy.utils import decorators
from snakypy.ansi import FG, BG, SGR
from snakypy.console import printer, entry, pick


__author__ = {
    'name': 'Snakypy Org',
    'email': 'contact.snakypy@gmail.com',
    'website': 'https://snakypy.github.io',
    'github': 'https://github.com/snakypy'
}
__version__ = '0.2.1'



from pathlib import Path


def create(*args, multidir=()):
    """
    Function that creates single or multiple directories using
    the "multidir" parameter.
    The "multidir" parameter must receive a tuple with the paths
    of the directories to be created.
    """

    # Create directory single.
    if args:
        for directory in args:
            path = Path(directory)
            path.mkdir(parents=True, exist_ok=True)
    # Create multiple directory.
    if len(multidir) > 0:
        for directory in multidir:
            path = Path(directory)
            path.mkdir(parents=True, exist_ok=True)



class NotSupportWindows(Exception):
    """
    A simple exception to show an unsupported error on Windows system.
    """

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return repr(self.text)



# from snakypy.utils import decorators
from snakypy.ansi import NONE, FG, BG, SGR


def attr_foreground_background_sgr(*args):
    """
    Checks if the attributes of the functions that the foreground
    and background parameters are in accordance with their respective class.

    Arguments:
        args {str} -- It receives a certain number of arguments with an ansi code value.
    """
    if args[0] and args[0] not in FG.__dict__.values():
        msg = 'Attribute invalid in parameter "foreground". Must receive from FG class.'
        raise AttributeError(msg)
    if args[1] and args[1] not in BG.__dict__.values():
        msg = 'Attribute invalid in parameter "background". Must receive from BG class.'
        raise AttributeError(msg)
    if args[2] and args[2] not in SGR.__dict__.values():
        msg = 'Attribute invalid in parameter "sgr". Must receive from SGR class.'
        raise AttributeError(msg)


# @decorators.use_unix_system
def printer(*args, foreground='', background='', sgr='',
            sep=' ', end='\n', file=None, flush=False):
    """A function that allows you to print colored text on the terminal.

    >>> from snakypy import printer, FG, BG, SGR
    >>> printer('Hello, World!', foreground=FG.BLACK, background=BG.WHITE, sgr=SGR.UNDERLINE)
    >>> printer('Hello, World!', foreground=FG.MAGENTA, sgr=SGR.UNDERLINE)

    Keyword Arguments:
        **foreground {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.FG" for the foreground \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        **background {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.BG" for the background \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        **sgr {str}** -- This named argument should optionally receive \
                     an object of class "snakypy.ansi.SGR" for the effect \
                     of the text. This object will be text with ansi code. \
                     (default: '')

        **sep {str}** -- Separator between printer function objects. (default: {' '}) \

        **end {str}** -- Responsible for skipping a line after printing is finished. \
                         (default: '[bar]n')
    """

    attr_foreground_background_sgr(foreground, background, sgr)

    lst = []
    for i in range(len(args)):
        lst.append(args[i])
    text = ' '.join(map(str, lst))

    return print(f'{NONE}{sgr}{foreground}{background}{text}{NONE}',
                 sep=sep, end=end, file=file, flush=flush)


# @decorators.use_unix_system
def entry(text, *, foreground='', background='', sgr='', jump_line='\n> '):
    """
    This function is derived from the input, but with the option of
    coloring it and some different formatting.
    Note: If you use Windows, the coloring option will not work.

    >>> from snakypy import entry, FG
    >>> entry("What's your name?", foreground=FG.QUESTION)
    >>> entry("What's your name?", foreground=FG.BLUE)
    >>> entry("What's your name?", foreground=FG.GREEN)

    Arguments:
        **text {object}** -- Argument must receive an object

    Keyword Arguments:

        **foreground {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.FG" for the foreground \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        background {str} -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.BG" for the background \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        **sgr {str}** -- This named argument should optionally receive \
                         an object of class "snakypy.ansi.SGR" for the effect \
                         of the text. This object will be text with ansi code. \
                         (default: '')

        **jump_line {str}** -- Named argument that makes the action of skipping a line \
                           and adding a greater sign to represent an arrow. You change \
                           that argument to your liking. (default: '[bar]n> ') \

    """

    attr_foreground_background_sgr(foreground, background, sgr)

    try:
        return input(f'{NONE}{sgr}{foreground}{background}{text}{jump_line}{NONE}')
    except KeyboardInterrupt:
        print(f'\n{FG.WARNING} Aborted by user.{NONE}')


def pick_options(title, options, answer, *, colorful=False, index=False, lowercase=False):
    if not colorful:
        FG.QUESTION = ''
        FG.GREEN = ''
        FG.MAGENTA = ''
        FG.CYAN = ''
        FG.ERROR = ''
        FG.WARNING = ''

    printer(title, '(Ctrl+C to Cancel)', foreground=FG.QUESTION)
    count = 1
    for option in options:
        print(f'{FG.GREEN}[{count}] {FG.MAGENTA}{option}{NONE}')
        count += 1
    try:
        pos = int(input(f'{FG.CYAN}{answer} {NONE}')) - 1
        assert pos > -1
        if index and lowercase:
            return pos, options[pos].lower()
        elif index and not lowercase:
            return pos, options[pos]
        if lowercase:
            return options[pos].lower()
        return options[pos]
    except Exception:
        printer('Option invalid!', foreground=FG.ERROR)
        return False
    except KeyboardInterrupt:
        printer('Canceled by user.', foreground=FG.WARNING)
        return


def pick(title, options: list, *,
         answer='Answer:',
         index=False,
         colorful=False,
         lowercase=False):
    """Function that creates a menu of options in the terminal.

    >>> from snakypy import pick
    >>> title = 'What is your favorite programming language?'
    >>> options = ['C', 'C++', 'Java', 'Javascript', 'Python', 'Ruby']
    >>> pick(title, options, lowercase=True)

    **output:**

    .. code-block:: shell

        What is your favorite programming language? (Ctrl+C to Cancel)
        [1] C
        [2] C++
        [3] Java
        [4] Javascript
        [5] Python
        [6] Ruby
        Answer: 5
        'python'

    Arguments:
        **title {str}** - You should receive a text that will be the \
                       title or the question with meaning in the alternatives.

        **options {list}** - You should receive a list with certain elements \
                          that will be part of the menu options.

    Keyword Arguments:
        **answer {str}** -- The text that will be shown before entering the answer. \
                        You can change to your language. (default: {'Answer:'})

        **index {bool}** -- This argument for True, will return a tuple, where element 0, \
                        will be the index of the option that the user chose, and \
                        element 1 of the tuple, will be the name of the choice option. \
                        Remember that the indexing of the menu is not the same as the \
                        list of options because it starts with zero (0). \
                        (default: {False})

        **colorful {bool}** -- If it has True, the menu color will be active, but it only \
                           works if it is on a UNIX system, as the color uses Ansi Color. \
                           If have Windows, no effect will appear. \
                           (default: {False})

        **lowercase {bool}** -- If set to True, the text value returned from the chosen \
                            option will be lowercase.
    """

    if not type(options) is list:
        raise TypeError('You must enter a list in the argument: options')

    if len(title) == 0:
        raise TypeError('The title cannot contain an empty element. Approached.')

    for option in options:
        if len(option) == 0:
            raise TypeError('The list cannot contain an empty element. Approached.')

    try:
        while True:
            option = pick_options(title,
                                  options,
                                  answer=answer,
                                  index=index,
                                  colorful=colorful)
            if option or option is None:
                break
        return option
    except Exception:
        raise Exception('An unexpected error occurs when using pick')


def billboard(text, foreground='', background='', ret_text=False):
    """
    Creates a Billboard in the terminal.

    >>> from snakypy.console import billboard
    >>> from snakypy import FG, BG
    >>> billboard('Hello, Snakypy!')
    >>> billboard('Hello, Snakypy!', foreground=FG.BLUE, background=BG.WHITE)

    Arguments:
        **text {str}** -- Any text must be informed.

    Keyword Arguments:
        **foreground {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.FG" for the foreground \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        **background {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.BG" for the background \
                            color of the text. This object will be text with ansi code. \
                            (default: '')

        **ret_text {bool}** -- Receives a Boolean value. If the value is True, it will only \
                               return the text. If the value is False, it will resume printing.

    Returns:
        **[str]** -- The text informed in billboard form.
    """
    import pyfiglet
    import snakypy

    banner = pyfiglet.figlet_format(text)
    if ret_text:
        return banner
    return snakypy.printer(banner, foreground=foreground, background=background)


# --------------------------------------------
# Function that does not use pyfliglet package
# --------------------------------------------
# def billboard(text, foreground='', background=''):
#     from snakypy.utilities.utilities import is_tool
#     from shutil import which
#     from snakypy.console import cmd
#
#     if which('figlet') is not None:
#         return cmd(f'figlet {text}', verbose=True)
#     return print(text)


def cmd(command, *args, shell=True, universal_newlines=True, ret=False, verbose=False):
    """
    Function that uses the subprocess library with Popen.
    The function receives a command as an argument and shows
    execution in real time.

    >>> from snakypy.console import cmd
    >>> command = 'git clone https://github.com/snakypy/snakypy.git'
    >>> cmd(command, verbose=True)

    Arguments:
        **command {str}** -- Must inform the command to be executed.

    Keyword Arguments:
        **shell {bool}** -- Receives a Boolean value. If it has False, the command must be \
                        in list where the command space is split. \
                        **E.g:** ['ls', '/bin']. \
                        If the value is True, the command can be stored in a string \
                        normal. **E.g:** 'ls /bin'

        **ret {bool}** -- The default value is False, however if it is set to True it will \
                      return a code status of the command output, where the code 0 (zero), \
                      is output without errors. \

        **verbose {bool}** -- The default value is False, if you change it to True, the command \
                          will show in real time the exit at the terminal, if there is an exit. \
    """
    from subprocess import Popen, PIPE

    process = Popen(command, shell=shell, stdout=PIPE,
                    universal_newlines=universal_newlines)
    if verbose:
        for line in iter(process.stdout.readline, ''):
            print(NONE, *args, line.rstrip(), NONE)
    if ret:
        r = process.poll()
        return r


def credence(app_name, app_version, app_url, data: dict, foreground=''):
    """
    Print project development credits.

    >>> from snakypy.console import credence
    >>> data = {
        "credence": [
            {
                "my_name": "William Canin",
                "email": "example@domain.com",
                "website": "http://williamcanin.me",
                "locale": "Brazil - SP"
            },
            {
                "my_name": "Maria Canin",
                "email": "example@domain.com",
                "locale": "Brazil - SP"
            }
        ]
    }
    >>> credence('Snakypy', '0.1.0', 'https://github.com/snakypy/snakypy', data)

    **output:**

    .. code-block:: shell

        ---------------------------------------------------------
                       Snakypy - Version 0.1.0
        ---------------------------------------------------------

                              Credence:

                        My Name: William Canin
                      Email: example@domain.com
                   Website: http://williamcanin.me
                         Locale: Brazil - SP

                         My Name: Maria Canin
                      Email: example@domain.com
                         Locale: Brazil - SP

        ---------------------------------------------------------
                    Snakypy © 2020 - All Right Reserved.
                Home: https://github.com/snakypy/snakypy
        ---------------------------------------------------------

    Arguments:
        app_name {str} -- Put application name.

        app_version {str} -- Application version.

        app_url {str} -- Application or website url.

        data {dict} -- You must receive a dictionary containing a key called "credence".
                       E.g: data = {'credence': []}

    Keyword Arguments:
        **foreground {str}** -- This named argument should optionally receive \
                            an object of class "snakypy.ansi.FG" for the foreground \
                            color of the text. This object will be text with ansi code. \
                            (default: '')
    """

    from datetime import date

    try:

        if type(data) is not dict:
            msg = f'>>> The function "{credence.__name__}" ' \
                  'must take a dictionary as an argument.'
            raise Exception(msg)

        printer(f'{57 * "-"}'.center(75), foreground=foreground)
        printer(f'{app_name} - Version {app_version}'.center(70),
                foreground=foreground)
        printer(f'{57 * "-"}\n'.center(75), foreground=foreground)
        printer(f'Credence:\n'.center(70), foreground=foreground)
        for item in data['credence']:
            for key, value in item.items():
                printer(f'{key.title().replace("_", " ")}: {value}'.center(70),
                        foreground=foreground)
            print()
        printer(f'{57 * "-"}'.center(75), foreground=foreground)
        printer(f'{app_name} © {date.today().year} - All Right Reserved.'.center(70),
                foreground=foreground)
        printer(f'Home: {app_url}'.center(70), foreground=foreground)
        printer(f'{57 * "-"}'.center(75), foreground=foreground)
    except KeyError:
        msg = "The 'credence' key was not found." \
              "Enter a dictionary containing a 'credits' key."
        raise KeyError(msg)


def loading(set_time=0.030, bar=False, header='[Loading]', foreground=''):
    """Function will show animated logging in percentage and bar style.

    >>> from snakypy.console import loading
    >>> loading()

    Using bar instead of percentage and setting the time (Default: set_time=0.030):

    >>> loading(set_time=0.20, bar=True)

    Keyword Arguments:

        **set_time {float}** -- Time when the animation will last. (default: {0.030})

        **bar {bool}** -- If True, the animation will be barred and not percentage. (default: {False})

        **header {str}** -- Modifies the animation header (default: {'[Loading]'})
    """

    import time
    import sys

    # denying_win(foreground)
    printer(header, foreground=foreground)
    try:
        if bar:
            for i in range(0, 100):
                time.sleep(set_time)  # 5 seconds
                width = (i + 1) / 4
                bar = f"[{'#' * int(width)}{' ' * (25 - int(width))}]"
                sys.stdout.write(u"\u001b[1000D" + bar)
                sys.stdout.flush()
            print()
            return
        for i in range(0, 100):
            time.sleep(set_time)
            sys.stdout.write(u"\u001b[1000D")
            sys.stdout.flush()
            # time.sleep(0.1)
            sys.stdout.write(f'{str(i + 1)}%')
            sys.stdout.flush()
        print()
        return
    except KeyboardInterrupt:
        printer('\nCanceled by user.', foreground=FG.WARNING)
        return


__all__ = ['pick', 'entry', 'printer', 'billboard', 'cmd', 'credence', 'loading']



from os.path import exists


def read(file_path, split=False):
    """Reads a text file.

    >>> import snakypy
    >>> file = '/tmp/my_file.txt'
    >>> snakypy.file.read(file)

    Arguments:
        **file_path {str}** -- You must receive the full/absolute file path.

    Keyword Arguments:
        **split {bool}** -- If this option is True, a list will be returned where \
        the breaks will be made using line skips. (default: {False})

    Returns:
        **[str|list]** -- By default it returns a string. If the option \
        split=True, a list of line breaks will be returned.
    """
    try:
        with open(file_path) as f:
            if split:
                return f.read().split('\n')
            return f.read()
    except FileNotFoundError as err:
        raise FileNotFoundError(f'>>> File "{file_path}" does not exist. {err}')


def create(content, file_path, force=False):
    """
    Create a text file.

    >>> import snakypy
    >>> snakypy.file.create('My file', '/tmp/file.txt')
    >>> snakypy.file.create('My file', '/tmp/file.txt', force=True)

    Arguments:
        **content {str}** -- Reports a text or an object containing a text.

        **file_path {str}** -- You must receive the full/absolute file path.

    Keyword Arguments:
        **force {bool}** -- Use the True option if you want to overwrite the existing file. \
                        (default: {False})

    Returns:
        **[bool]** -- If everything went well, it will return True.
    """

    if not force and exists(file_path):
        raise FileExistsError(f'>>> The file {file_path} already exists, use force=True.')
    else:
        try:
            with open(file_path, 'w') as f:
                f.write(content)
                return True
        except Exception as err:
            raise Exception(f'>>> There was an error creating the file: {err}')


__all__ = ['read', 'create']



from subprocess import check_output
from snakypy.utils import decorators


@decorators.use_unix_system
def shell():
    """
    Function to get the currently activated shell.
    Note: snakypy.catch.extension(file)

    >>> import snakypy
    >>> snakypy.catch.shell()

    **output:**

    .. code-block:: shell

        'zsh'

    Returns:
        [str] -- Returns the name of the current shell.
    """

    from sys import platform

    if not platform.startswith('win'):
        s = check_output("echo $0", shell=True, universal_newlines=True)
        lst = s.strip('\n').strip('').split('/')
        return lst[2]


def extension(filename, first_dot=False):
    """Get a file extension

    >>> import snakypy
    >>> file = '/tmp/file.tar.gz'
    >>> snakypy.catch.extension(file)
    >>> snakypy.catch.extension(file, first_dot=True)

    **output:**

    .. code-block:: shell

        'gz'
        '.tar.gz'

    Arguments:
        **filename {str}** -- Receives the file name or its full path

    Keyword Arguments:
        **first_dot {bool}** -- If it is True, I return the extension from the first \
                            point found, that is, if the file has more than one point, \
                            the first one will be the capture point, otherwise it will \
                            take the last point found. (Default: {False})

    Returns:
        [object] -- Returns a string containing the extension or None.
    """

    if first_dot:
        import re
        m = re.search(r'(?<=[^/\\]\.).*$', filename)
        if not m:
            return None
        ext = m.group(0)
        return ext
    else:
        from os.path import splitext
        ext = splitext(filename)[1]
        if ext:
            return ext
        return None


__all__ = ['shell', 'extension']



import json
from os.path import exists


def read(file_path):
    """
    Function that reads JSON configuration file and returns data.

    >>> import snakypy
    >>> file = '/tmp/file.json'
    >>> snakypy.json.read(file)

    Arguments:
        **file_path {str}** -- You must receive the full/absolute file path.

    Returns:
        [dict] -- If the file is found it will return a dictionary
    """

    try:
        with open(file_path) as f:
            data = json.load(f)
        return data
    except FileNotFoundError as err:
        raise FileNotFoundError(f'>>> File not found {err}')
    except Exception:
        raise Exception(f'>>> There was an error reading the file: {file_path}')


def create(dictionary, file_path, force=False):
    """
    Create a JSON file through a dictionary.

    >>> import snakypy
    >>> dic = {'msg': 'Hello, Snakypy!'}
    >>> snakypy.json.create(dic, '/tmp/file.json')
    >>> snakypy.json.create(dic, '/tmp/file.json', force=True)

    Arguments:
        **dictionary {dict}** -- [description]

        **file_path {str}** -- You must receive the full/absolute file path.

    Keyword Arguments:
        **force {bool}** -- Use the True option if you want to overwrite the existing file. \
                            (default: {False})

    Returns:
        **[bool]** -- If everything went well, it will return True.
    """
    from os.path import splitext

    if splitext(file_path)[1] != '.json':
        raise Exception('The JSON file extension was not explicit.')
    if not force and exists(file_path):
        raise FileExistsError(f'>>> The file {file_path} already exists, use force=True.')
    else:
        try:
            if type(dictionary) is dict:
                with open(file_path, 'w') as f:
                    json.dump(dictionary, f, indent=4, separators=(',', ': '))
                    return True
            return False
        except Exception as err:
            raise Exception(f'>>> There was an error creating the file. {err}')


def update(file_path, content: dict):
    """
    Function to update json file. The "snakypy.json.read" function depends on
    reading a json file.

    >>> import snakypy
    >>> data = snakypy.json.read('/tmp/file.json')
    >>> data['msg'] = 'Olá, Snakypy!'
    >>> snakypy.json.update('/tmp/file.json', data)

    Arguments:
        **file_path {str}** -- You must receive the full/absolute file path.

        **content {dict}** -- You should receive a dictionary with the updated data \
                              already.
    Returns:
        **[bool]** -- If everything went well, it will return True.
    """
    try:
        if type(content) is dict:
            with open(file_path, 'w') as f:
                json.dump(content, f, indent=2, separators=(',', ': '))
            return True
        return False
    except Exception as err:
        msg = f'>>> Something unexpected happened while updating {file_path}'
        raise Exception(msg, err)


__all__ = ['read', 'create']



from os import walk, remove
from os.path import join
from shutil import rmtree
from threading import Thread


def cleaner(directory, *file, level=None):
    """
    **DANGER!** A function for cleaning objects and folders on the system.

    Arguments:
        **directory {str}** -- Directory where are the files to be destroyed

        ***file** -- Enter an N file name number (Optional)

    Keyword Arguments:
        **level {int}** -- This option receives 3 values, they are: \

                           Value 0 = If this value is set, the function revokes the \
                           unitary file exclusion option, that is, this option will \
                           exclude all files at the root of the informed directory.

                           Value 1 = If this value is set, the function revokes the unitary \
                           file exclusion option as well, however, it will exclude all \
                           subdirectories of the root directory, except the files contained \
                           in the root.

                           Value None = If this value is set, the function must receive at least \
                           one file name to be deleted. Can pass as many files as you want.

                           (default: {None})

    """

    data = next(walk(directory))

    #: DANGER!
    if level == 0:
        for f in data[2]:
            remove(join(data[0], f))
        return 0

    #: DANGER!
    if level == 1:
        # r: root, r: directory, f: files
        for r, d, f in walk(directory, topdown=False):
            for item in d:
                t = Thread(target=rmtree(join(r, item)), args=())
                t.start()
        return 1

    try:
        if file:
            for f in file:
                remove(join(data[0], f))
            return
    except FileNotFoundError as err:
        msg = '>>> There was an error removing the files'
        raise FileNotFoundError(msg, err)


__all__ = ['cleaner']
